# -*- coding: utf-8 -*-
import socket
import os
import imp


DEFAULTS = dict(
    project='.',
    env='%(project)s/.env',
    production=False,
    requirements='config/requirements.txt',
    virtualenv_params='',
    settings_path='config.settings',
    run='echo "add run command to config"',
    bootstrap_script='bootstrap.sh',
    cloud_type='vps',
)


def set_defaults(config):
    for k, v in DEFAULTS.iteritems():
        if k not in config:
            config[k] = v
    return config


def make_params(config):
    for k, v in config.iteritems():
        if str == type(v) and '%(' in v:
            config[k] = v % config
    return config


def make_config(config):
    config = set_defaults(config)
    config = make_params(config)
    return config


def get_config(module, name=None):
    if str == type(module):
        if module.endswith('.py'):
            module = imp.load_source('', module)
        else:
            module = __import__(module)
    configs = dir(module)
    if 'FABRICATOR_ENV' in os.environ:
        name = os.environ['FABRICATOR_ENV']
    safe = False if name else True
    name = name if name else socket.gethostname()
    if name and name in configs:
        config = getattr(module, name)
        config['target'] = name
        return make_config(config)
    else:
        if safe:
            return make_config({})
        else:
            config_names = ', '.join(filter(lambda c: not c.startswith('__'), dir(module)))
            raise Exception('Config with name <%s> not found in module <%s>. Available configs: <%s>' % (name, module.__name__, config_names))


def fill_fabric_env(module, name=None):
    from fabric.operations import local as fabric_local, run as fabric_run
    from fabric.state import env
    config = get_config(module, name)
    env.update(config)
    env.method = fabric_run if 'ssh_host' in config else fabric_local

    if 'ssh_host' in config:
        host = config.get('ssh_host', [])
        if list == type(host):
            env.hosts = host
        else:
            env.hosts = [host]
        env.user = config.get('ssh_user', None)

        if 'ssh_pass' in config:
            env.password = config['ssh_pass']

        if 'ssh_key' in config:
            env.key_filename = config['ssh_key']



# -*- coding: utf-8 -*-

test_config = dict(
    ssh_host='127.0.0.1',
    ssh_pass='pass'
)

digitaldemiurge = dict(
    ssh_host='10.10.10.2',
    ssh_user='vagrant',
    ssh_key='/home/vitalis/.vagrant.d/insecure_private_key',
)



# -*- coding: utf-8 -*-
import unittest
from fabricator.config import fill_fabric_env
from fabric.state import env


class ConfigTestCase(unittest.TestCase):
    def test_get_config(self):
        fill_fabric_env(env, 'test_project.test_config', 'test_config')


# -*- coding: utf-8 -*-
from fabric.context_managers import prefix, cd, settings
from fabric.state import env
from fabric.decorators import task
from fabric.operations import local as fabric_local, run as fabric_run, sudo
from contextlib import contextmanager
from string import Template
from fabric import colors
from fabric.contrib.console import confirm
import webbrowser
import os


@contextmanager
def virtualenv():
    with cd(env.project):
        with prefix('source %(env)s/bin/activate' % env):
            yield


@task
def sh(cmd, is_cd=True):
    if is_cd:
        with cd(env.project):
            env.method(cmd)
    else:
        env.method(cmd)


@task
def vsh(cmd):
    with virtualenv():
        sh(cmd, False)


@task
def push(message='Blank message'):
    fabric_local('git commit -a -m "%s"' % message)
    fabric_local('git push')


@task
def run(cmd=None):
    if cmd:
        vsh(env['run_' + cmd])
    else:
        vsh(env.run)


@task
def pull():
    sh('git pull')


@task
def update():
    push()
    pull()


# ====================================================================================================
# Django
# ====================================================================================================
@task
def dj(cmd):
    vsh('export FABRICATOR_ENV="%s" && python manage.py %s --settings %s' % (env.target, cmd, env.settings_path))


@task
def sync():
    dj('syncdb --noinput')


@task
def static():
    dj('collectstatic --noinput')


@task
def requirements():
    with virtualenv():
        sh('pip install -r %(requirements)s' % env)


@task
def virtualenv_init():
    sh('virtualenv %(virtualenv_params)s %(env)s' % env)


@task
def init():
    if env.production:
        sh('mkdir -p %s' % env.project, False)
        sh('git clone %s .' % env.repository)
    virtualenv_init()
    requirements()


@task
def tests(cv=False):
    with settings(warn_only=True):
        update()
    params = '--with-coverage --cover-html' if cv else ''
    vsh('nosetests -s %s' % params)


@task(alias='make_admin')
def create_superuser(user='admin', email='admin@admin.com'):
    dj('createsuperuser --username %s --email %s' % (user, email))


@task(alias='mgr')
def south_migrate(app=''):
    dj('migrate %s' % app)


@task(alias='mgr-list')
def south_migrate_list():
    dj('migrate --list')


@task(alias='mgr-init')
def south_migration_initial(app):
    dj('schemamigration --initial %s' % app)


@task(alias='mgr-auto')
def south_migration_auto(app):
    dj('schemamigration --auto %s' % app)


@task
def fixture(name):
    dj('generate_fixture %s.test_data' % name)


@task(alias='import_dump_pg')
def import_dump_postgresql(dump):
    env.method('PGPASSWORD=%(PASSWORD)s psql -U %(USER)s %(NAME)s < ' % env.postgresql + dump)


# ====================================================================================================
# Crontabs
# ====================================================================================================
@task(alias='cron-add')
def crontab_add():
    dj('crontab add')


@taks(alias='cron-rm')
def crontab_remove():
    dj('crontab remove)


@task(alias='cron-list')
def crontab_list():
    sh('crontab -l')

# ====================================================================================================
# Supervisor
# ====================================================================================================
@task(alias='sv_cmd')
def supervisor_cmd(cmd):
    sudo('supervisorctl %s' % cmd)


@task(alias='sv_status')
def supervisor_status():
    supervisor_cmd('status')


@task(alias='sv_tail')
def supervisor_tail(service):
    supervisor_cmd('tail %s' % service)


@task(alias='sv_stop')
def supervisor_stop(service):
    supervisor_cmd('stop %s' % service)


@task(alias='sv_start')
def supervisor_start(service):
    supervisor_cmd('start %s' % service)


@task(alias='sv_restart')
def supervisor_restart(service):
    supervisor_cmd('restart %s' % service)


# ====================================================================================================
# Mongodb
# ====================================================================================================
@task(alias='mdb_cmd')
def mongodb_run_command(cmd):
    env.method('echo "%s" | mongo' % cmd)


@task(alias='mdb_web')
def mongodb_open_web_console():
    webbrowser.open('http://%s:28017' % env.ssh_host)


@task(alias='mdbrs_init')
def mongodb_replicaset_init():
    mongodb_run_command('rs.initiate();')


@task(alias='mdbrs_status')
def mongodb_replicaset_status():
    mongodb_run_command('rs.status();')


@task(alias='mdbrs_add')
def mongodb_replicaset_add(host):
    mongodb_run_command("rs.add('%s');" % host)


@task(alias='mdbrs_info')
def mongodb_replicaset_info():
    mongodb_run_command('rs.isMaster();')


@task(alias='mdbrs_conf')
def mongodb_replicaset_conf():
    mongodb_run_command('rs.conf();')


# ====================================================================================================
# Remote servers
# ====================================================================================================
def get_script(filename, **kwargs):
    template = open('files/' + filename).read()
    if kwargs:
        return Template(template).substitute(**kwargs)
    else:
        return template


@task(alias='srv_launch')
def server_launch(user=None, password=None):
    if 'vagrant' == env.cloud_type:
        server = env.server
        dir = '/tmp/%s' % server
        if not os.path.exists(dir):
            os.mkdir(dir)
        if os.path.exists('%s/Vagrantfile' % dir):
            print colors.red('Machine <%s> exists.' % server)

        fabric_local('cd %s && echo "%s" > Vagrantfile' % (dir, get_script('Vagrantfile',
            host=server,
            ip_addr=env.ssh_host
        )))
        fabric_local('cd %s && vagrant up' % dir)
    elif 'vps' == env.cloud_type:
        env.user = user if user else env.user
        if password:
            env.password = password
            env.key_filename = None

    bootstrap_script = get_script(
        env.bootstrap_script,
        puppet_source=env.repository,
        user=env.ssh_user,
        sshd_config=get_script('sshd_config'),
        vcs_known_hosts=get_script('vcs_known_hosts'),
        vcs_deploy_public=get_script('vcs_deploy_public'),
        vcs_deploy_private=get_script('vcs_deploy_private'),
        environment=env.target,
    )

    fabric_run('echo "%s" > bootstrap.sh' % bootstrap_script)
    sudo('chmod +x bootstrap.sh && ./bootstrap.sh')


@task(alias='srv_apply')
def server_apply():
    sudo('cd /etc/puppet && git pull')
    sudo('cd /etc/puppet && git submodule update --init --recursive')
    sudo('puppet apply /etc/puppet/manifests/%s.pp' % env.target)


@task
def ssh(cmd=None):
    if 'vagrant' == env.cloud_type:
        fabric_local('cd /tmp/%s && vagrant ssh' % env.server)
    elif 'vps' == env.cloud_type:
        cmd = ' -t %s' % cmd if cmd else ''
        cmd_str = 'ssh %(ssh_user)s@%(ssh_host)s -i %(ssh_key)s' % env
        cmd_str += cmd
        fabric_local(cmd_str)


@task(alias='shell')
def django_shell():
    cmd = '"export FABRICATOR_ENV=%s && cd %s && %s/bin/python manage.py shell --settings %s"' % (env.target, env.project, env.env, env.settings_path)
    ssh(cmd)


@task(alias='srv_reboot')
def server_reboot():
    if 'vagrant' == env.cloud_type:
        fabric_local('cd /tmp/%s && vagrant reload' % env.server)


@task(alias='srv_stop')
def server_stop():
    if 'vagrant' == env.cloud_type:
        fabric_local('cd /tmp/%s && vagrant halt' % env.server)


@task(alias='srv_start')
def server_start():
    if 'vagrant' == env.cloud_type:
        fabric_local('cd /tmp/%s && vagrant up' % env.server)


@task(alias='srv_destroy')
def server_destroy():
    if confirm(colors.red('Destroy machine <%s> (Danger operation!!!!!!!)' % env.server)):
        if 'vagrant' == env.cloud_type:
            dir = '/tmp/%s' % env.server
            fabric_local('cd %s && vagrant destroy' % dir)
            fabric_local('rm -rf %s' % dir)



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

VERSION = (0, 1, 30)
__version__ = VERSION
__versionstr__ = '.'.join(map(str, VERSION))

setup(
    name='fabricator',
    version=__versionstr__,
    description="Multi user configuration and default fabric targets for you python project",
    author='SevenQuark',
    author_email='info@sevenquark.com',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,

    license='MIT license',
    url='https://github.com/SevenQuark/fabricator',
#    long_description = open('README.rst').read(),
    requires=['Fabric (>= 1.4)'],

    classifiers=(
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ),
)



# -*- coding: utf-8 -*-

# Config for remote server with ssh key authentication
hostname1 = dict(
    ssh_host='111.222.333.444',
    ssh_user='user',
    ssh_key='/path/to/private/ssh/key/id_rsa'
)

# Config for remote server with ssh password authentication
hostname12 = dict(
    ssh_host='111.222.333.444',
    ssh_user='user',
    ssh_pass='password'
)


digitaldemiurge = dict(
    ssh_host='10.10.10.2',
    ssh_user='vagrant',
    ssh_key='/home/vitalis/.vagrant.d/insecure_private_key'
)
